@@ -31,7 +31,7 @@
 #include <windows.h>
 #endif
 
-#undef PACKAGE
+#undef PACKAGE_NAME
 #include "../../config.h"		  /* for #define OLD_PERL */
 #include "xchat-plugin.h"
 
@@ -1284,7 +1284,7 @@ xchat_plugin_get_info (char **name, char **desc, char **version,
 {
 	*name = "Perl";
 	*desc = "Perl scripting interface";
-	*version = VERSION;
+	*version = PACKAGE_VERSION;
 	if (reserved)
 		*reserved = NULL;
 }
@@ -1310,7 +1310,7 @@ xchat_plugin_init (xchat_plugin * plugin_handle, char **plugin_name,
 
 	*plugin_name = "Perl";
 	*plugin_desc = "Perl scripting interface";
-	*plugin_version = VERSION;
+	*plugin_version = PACKAGE_VERSION;
 
 	xchat_hook_command (ph, "load", XCHAT_PRI_NORM, perl_command_load, 0, 0);
 	xchat_hook_command (ph, "unload", XCHAT_PRI_NORM, perl_command_unload, 0,
