@@ -1,8 +1,8 @@
 /***************************************************************************
-                          tclplugin.c  -  Tcl plugin for xchat 1.9.x / 2.x.x
-                          --------------------------------------------------
+                           tclplugin.c  -  Tcl plugin for xchat 1.9.x / 2.x.x
+                           -------------------------------------------------s
     begin                : Sat Nov 19 17:31:20 MST 2002
-    copyright            : Copyright 2002-2004 Daniel P. Stasinski
+    copyright            : Copyright 2002-2005 Daniel P. Stasinski
     email                : mooooooo@avenues.org
  ***************************************************************************/
 
@@ -15,8 +15,9 @@
  *                                                                         *
  ***************************************************************************/
 
-#define VERSION "1.0.55"
+static char RCSID[] = "$Id: tclplugin.c,v 1.59 2005/08/11 02:37:55 mooooooo Exp $";
 
+#include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -40,10 +41,15 @@
 #include "tclplugin.h"
 #include "printevents.h"
 
+int xchat_plugin_deinit(void);
+void xchat_plugin_get_info(char **name, char **desc, char **version, void **reserved);
+
 static int nexttimerid = 0;
 static int nexttimerindex = 0;
 static timer timers[MAX_TIMERS];
 
+static char PVERSION[16];
+
 static int initialized = 0;
 static int reinit_tried = 0;
 static Tcl_Interp *interp = NULL;
@@ -100,11 +106,43 @@ static char inlinetcl[] = {
 "proc splitsrc { } {\n"
 "uplevel \"scan \\$_src \\\"%\\\\\\[^!\\\\\\]!%\\\\\\[^@\\\\\\]@%s\\\" _nick _ident _host\"\n"
 "}\n"
+
 "proc ::exit { } {\n"
 "puts \"Using 'exit' is bad\"\n"
 "}\n"
+
+"proc ::away { args } { return [eval [join [list getinfo $args away]]] }\n"
+"proc ::channel { args } { return [eval [join [list getinfo $args channel]]] }\n"
+"proc ::tab { args } { return [eval [join [list getinfo $args channel]]] }\n"
+"proc ::charset { args } { return [eval [join [list getinfo $args charset]]] }\n"
+"proc ::host { args } { return [eval [join [list getinfo $args host]]] }\n"
+"proc ::inputbox { args } { return [eval [join [list getinfo $args inputbox]]] }\n"
+"proc ::libdirfs { args } { return [eval [join [list getinfo $args libdirfs]]] }\n"
+"proc ::network { args } { return [eval [join [list getinfo $args network]]] }\n"
+"proc ::nickserv { args } { return [eval [join [list getinfo $args nickserv]]] }\n"
+"proc ::server { args } { return [eval [join [list getinfo $args server]]] }\n"
+"proc ::version { args } { return [eval [join [list getinfo $args version]]] }\n"
+"proc ::win_status { args } { return [eval [join [list getinfo $args win_status]]] }\n"
+"proc ::xchatdir { args } { return [eval [join [list getinfo $args xchatdir]]] }\n"
+"proc ::xchatdirfs { args } { return [eval [join [list getinfo $args xchatdirfs]]] }\n"
+
+"proc ::color { {arg {}} } { return \"\\003$arg\" }\n"
+"proc ::bold { } { return \"\\002\" }\n"
+"proc ::underline { } { return \"\\037\" }\n"
+"proc ::reverse { } { return \"\\026\" }\n"
+"proc ::reset { } { return \"\\017\" }\n"
+
+"proc ::__xctcl_errorInfo { } {\n"
+"      set text [split $::errorInfo \\n]\n"
+"      puts [string trim [join [lrange $text 0 [expr {[llength $text] - 4}]] \\n]]\n"
+"}\n"
 };
 
+static void NiceErrorInfo ()
+{
+    Tcl_Eval(interp, "::__xctcl_errorInfo");
+}
+
 static void Tcl_MyDStringAppend(Tcl_DString * ds, char *string)
 {
     Tcl_DStringAppend(ds, string, strlen(string));
@@ -312,7 +350,6 @@ static int insert_timer(int seconds, int count, const char *script)
     int x;
     int dummy;
     time_t now;
-    const char *errorInfo;
     int id;
 
     if (script == NULL)
@@ -325,8 +362,8 @@ static int insert_timer(int seconds, int count, const char *script)
     for (x = 1; x < MAX_TIMERS; x++) {
         if (timers[x].timerid == 0) {
             if (SourceInternalProc(id, "", script) == TCL_ERROR) {
-                errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
-                xchat_printf(ph, "\0039TCL plugin\003\tERROR (timer %d) %s \n", timers[x].timerid, errorInfo);
+                xchat_printf(ph, "\0039TCL plugin\003\tERROR (timer %d) ", timers[x].timerid);
+                NiceErrorInfo ();
                 return (-1);
             }
             timers[x].timerid = (nexttimerid++ % INT_MAX) + 1;
@@ -347,7 +384,6 @@ static void do_timer()
     xchat_context *origctx;
     time_t now;
     int index;
-    const char *errorInfo;
 
     if (!nexttimerindex)
         return;
@@ -360,8 +396,8 @@ static void do_timer()
     index = nexttimerindex;
     origctx = xchat_get_context(ph);
     if (EvalInternalProc(timers[index].procPtr, 0) == TCL_ERROR) {
-        errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
-        xchat_printf(ph, "\0039TCL plugin\003\tERROR (timer) %s \n", errorInfo);
+        xchat_printf(ph, "\0039TCL plugin\003\tERROR (timer %d) ", timers[index].timerid);
+        NiceErrorInfo ();
     }
     xchat_set_context(ph, origctx);
 
@@ -400,7 +436,6 @@ static int Server_raw_line(char *word[], char *word_eol[], void *userdata)
     int list_argc, proc_argc;
     const char **list_argv, **proc_argv;
     int private = 0;
-    const char *errorInfo;
 
     if (word[0][0] == 0)
         return XCHAT_EAT_NONE;
@@ -494,8 +529,8 @@ static int Server_raw_line(char *word[], char *word_eol[], void *userdata)
 
                 origctx = xchat_get_context(ph);
                 if (EvalInternalProc(proc_argv[1], 7, src, dest, cmd, rest, word_eol[1], proc_argv[0], myitoa(private)) == TCL_ERROR) {
-                    errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
-                    xchat_printf(ph, "\0039TCL plugin\003\tERROR (on %s %s) %s \n", cmd, proc_argv[0], errorInfo);
+                    xchat_printf(ph, "\0039TCL plugin\003\tERROR (on %s %s) ", cmd, proc_argv[0]);
+                    NiceErrorInfo ();
                 }
                 xchat_set_context(ph, origctx);
 
@@ -531,7 +566,6 @@ static int Print_Hook(char *word[], void *userdata)
     const char **list_argv, **proc_argv;
     Tcl_DString ds;
     int x;
-    const char *errorInfo;
 
     if (complete_level == MAX_COMPLETES)
         return XCHAT_EAT_NONE;
@@ -560,8 +594,8 @@ static int Print_Hook(char *word[], void *userdata)
                     Tcl_DStringAppend(&ds, "!*@", 3);
                     Tcl_DStringAppend(&ds, word[1], strlen(word[1]));
                     if (EvalInternalProc(proc_argv[1], 7, ds.string, word[2], xc[(int) userdata].event, word[4], "", proc_argv[0], "0") == TCL_ERROR) {
-                        errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
-                        xchat_printf(ph, "\0039TCL plugin\003\tERROR (on %s %s) %s \n", xc[(int) userdata].event, proc_argv[0], errorInfo);
+                        xchat_printf(ph, "\0039TCL plugin\003\tERROR (on %s %s) ", xc[(int) userdata].event, proc_argv[0]);
+                        NiceErrorInfo ();
                     }
                 } else {
                     if (xc[(int) userdata].argc > 0) {
@@ -569,8 +603,8 @@ static int Print_Hook(char *word[], void *userdata)
                             Tcl_DStringAppendElement(&ds, word[x]);
                     }
                     if (EvalInternalProc(proc_argv[1], 7, "", "", xc[(int) userdata].event, "", ds.string, proc_argv[0], "0") == TCL_ERROR) {
-                        errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
-                        xchat_printf(ph, "\0039Tcl plugin\003\tERROR (on %s %s) %s \n", xc[(int) userdata].event, proc_argv[0], errorInfo);
+                        xchat_printf(ph, "\0039Tcl plugin\003\tERROR (on %s %s) ", xc[(int) userdata].event, proc_argv[0]);
+                        NiceErrorInfo ();
                     }
                 }
 
@@ -669,8 +703,8 @@ static int tcl_timers(ClientData cd, Tcl_Interp * irp, int argc, const char *arg
             Tcl_DStringAppendElement(&ds, myitoa((long)timers[x].timerid));
             Tcl_DStringAppendElement(&ds, myitoa((long)timers[x].timestamp - now));
             Tcl_DStringAppendElement(&ds, timers[x].procPtr);
-	    Tcl_DStringAppendElement(&ds, myitoa((long)timers[x].seconds));
-	    Tcl_DStringAppendElement(&ds, myitoa((long)timers[x].count));
+            Tcl_DStringAppendElement(&ds, myitoa((long)timers[x].seconds));
+            Tcl_DStringAppendElement(&ds, myitoa((long)timers[x].count));
             Tcl_DStringEndSublist(&ds);
         }
     }
@@ -740,15 +774,14 @@ static int tcl_on(ClientData cd, Tcl_Interp * irp, int argc, const char *argv[])
     int list_argc, proc_argc;
     int id;
     const char **list_argv, **proc_argv;
-    const char *errorInfo;
 
     BADARGS(4, 4, " token label {script | procname ?args?}");
 
     id = (nextprocid++ % INT_MAX) + 1;
 
     if (SourceInternalProc(id, "_src _dest _cmd _rest _raw _label _private", argv[3]) == TCL_ERROR) {
-        errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
-        xchat_printf(ph, "\0039Tcl plugin\003\tERROR (on %s:%s) %s \n", argv[1], argv[2], errorInfo);
+        xchat_printf(ph, "\0039Tcl plugin\003\tERROR (on %s:%s) ", argv[1], argv[2]);
+        NiceErrorInfo ();
         return TCL_OK;
     }
 
@@ -885,7 +918,6 @@ static int tcl_alias(ClientData cd, Tcl_Interp * irp, int argc, const char *argv
     const char *help = NULL;
     int dummy;
     int id;
-    const char *errorInfo;
 
     BADARGS(3, 4, " name ?help? {script | procname ?args?}");
 
@@ -900,8 +932,8 @@ static int tcl_alias(ClientData cd, Tcl_Interp * irp, int argc, const char *argv
         id = (nextprocid++ % INT_MAX) + 1;
 
         if (SourceInternalProc(id, "_cmd _rest", argv[argc - 1]) == TCL_ERROR) {
-            errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
-            xchat_printf(ph, "\0039Tcl plugin\003\tERROR (alias %s) %s \n", argv[1], errorInfo);
+            xchat_printf(ph, "\0039Tcl plugin\003\tERROR (alias %s) ", argv[1]);
+            NiceErrorInfo ();
             return TCL_OK;
         }
 
@@ -1034,12 +1066,6 @@ static int tcl_raw(ClientData cd, Tcl_Interp * irp, int argc, const char *argv[]
     return TCL_OK;
 }
 
-static int tcl_getinfo(ClientData cd, Tcl_Interp * irp, int argc, const char *argv[])
-{
-    BADARGS(2, 2, " id");
-    Tcl_AppendResult(irp, xchat_get_info(ph, argv[1]), NULL);
-    return TCL_OK;
-}
 
 static int tcl_prefs(ClientData cd, Tcl_Interp * irp, int argc, const char *argv[])
 {
@@ -1063,58 +1089,49 @@ static int tcl_prefs(ClientData cd, Tcl_Interp * irp, int argc, const char *argv
     return TCL_OK;
 }
 
-static int tcl_server(ClientData cd, Tcl_Interp * irp, int argc, const char *argv[])
+static int tcl_info(ClientData cd, Tcl_Interp * irp, int argc, const char *argv[], char *id)
 {
+    char *result;
+    int max_argc;
     xchat_context *origctx, *ctx;
-    BADARGS(1, 2, " ?server|network|context?");
+
+    if (id == NULL) {
+        BADARGS(2, 3, " ?server|network|context? id");
+        max_argc = 3;
+    } else {
+        BADARGS(1, 2, " ?server|network|context?");
+        max_argc = 2;
+    }
 
     origctx = xchat_get_context(ph);
 
-    if (argc == 2) {
+    if (argc == max_argc) {
         ctx = xchat_smart_context(argv[1], NULL);
         CHECKCTX(ctx);
         xchat_set_context(ph, ctx);
     }
 
-    Tcl_AppendResult(irp, xchat_get_info(ph, "server"), NULL);
+    if (id == NULL)
+      id = argv[argc-1];
+
+    if ((result = xchat_get_info(ph, id)) == NULL)
+        result = "";
+
+    Tcl_AppendResult(irp, result, NULL);
 
     xchat_set_context(ph, origctx);
 
     return TCL_OK;
 }
 
-static int tcl_channel(ClientData cd, Tcl_Interp * irp, int argc, const char *argv[])
+static int tcl_me(ClientData cd, Tcl_Interp * irp, int argc, const char *argv[])
 {
-    xchat_context *origctx, *ctx;
-    BADARGS(1, 2, " ?server|network|context?");
-
-    origctx = xchat_get_context(ph);
-
-    if (argc == 2) {
-        ctx = xchat_smart_context(argv[1], NULL);
-        CHECKCTX(ctx);
-        xchat_set_context(ph, ctx);
-    }
-
-    Tcl_AppendResult(irp, xchat_get_info(ph, "channel"), NULL);
-
-    xchat_set_context(ph, origctx);
-
-    return TCL_OK;
+    return tcl_info(cd, irp, argc, argv, "nick");
 }
 
-static int tcl_version(ClientData cd, Tcl_Interp * irp, int argc, const char *argv[])
+static int tcl_getinfo(ClientData cd, Tcl_Interp * irp, int argc, const char *argv[])
 {
-    BADARGS(1, 1, "");
-    Tcl_AppendResult(irp, xchat_get_info(ph, "version"), NULL);
-    return TCL_OK;
-}
-
-static int tcl_xchatdir(ClientData cd, Tcl_Interp * irp, int argc, const char *argv[])
-{
-    BADARGS(1, 1, "");
-    Tcl_AppendResult(irp, xchat_get_info(ph, "xchatdir"), NULL);
-    return TCL_OK;
+    return tcl_info(cd, irp, argc, argv, NULL);
 }
 
 static int tcl_getlist(ClientData cd, Tcl_Interp * irp, int argc, const char *argv[])
@@ -1559,6 +1576,8 @@ static int tcl_users(ClientData cd, Tcl_Interp * irp, int argc, const char *argv
         Tcl_DStringAppendElement(&ds, "nick");
         Tcl_DStringAppendElement(&ds, "host");
         Tcl_DStringAppendElement(&ds, "prefix");
+        Tcl_DStringAppendElement(&ds, "away");
+        Tcl_DStringAppendElement(&ds, "lasttalk");
         Tcl_DStringEndSublist(&ds);
 
         while (xchat_list_next(ph, list)) {
@@ -1566,6 +1585,8 @@ static int tcl_users(ClientData cd, Tcl_Interp * irp, int argc, const char *argv
             Tcl_DStringAppendElement(&ds, (const char *) xchat_list_str(ph, list, "nick"));
             Tcl_DStringAppendElement(&ds, (const char *) xchat_list_str(ph, list, "host"));
             Tcl_DStringAppendElement(&ds, (const char *) xchat_list_str(ph, list, "prefix"));
+            Tcl_DStringAppendElement(&ds, myitoa((long)xchat_list_int(ph, list, "away")));
+            Tcl_DStringAppendElement(&ds, myitoa((long)xchat_list_time(ph, list, "lasttalk")));
             Tcl_DStringEndSublist(&ds);
         }
 
@@ -1581,6 +1602,48 @@ static int tcl_users(ClientData cd, Tcl_Interp * irp, int argc, const char *argv
     return TCL_OK;
 }
 
+static int tcl_notifylist(ClientData cd, Tcl_Interp * irp, int argc, const char *argv[])
+{
+    xchat_list *list;
+    Tcl_DString ds;
+
+    BADARGS(1, 1, "");
+
+    Tcl_DStringInit(&ds);
+
+    list = xchat_list_get(ph, "notify");
+
+    if (list != NULL) {
+
+        Tcl_DStringStartSublist(&ds);
+        Tcl_DStringAppendElement(&ds, "nick");
+        Tcl_DStringAppendElement(&ds, "flags");
+        Tcl_DStringAppendElement(&ds, "on");
+        Tcl_DStringAppendElement(&ds, "off");
+        Tcl_DStringAppendElement(&ds, "seen");
+        Tcl_DStringEndSublist(&ds);
+
+        while (xchat_list_next(ph, list)) {
+            Tcl_DStringStartSublist(&ds);
+            Tcl_DStringAppendElement(&ds, (const char *) xchat_list_str(ph, list, "nick"));
+            Tcl_DStringAppendElement(&ds, (const char *) xchat_list_str(ph, list, "flags"));
+            Tcl_DStringAppendElement(&ds, myitoa((long)xchat_list_time(ph, list, "on")));
+            Tcl_DStringAppendElement(&ds, myitoa((long)xchat_list_time(ph, list, "off")));
+            Tcl_DStringAppendElement(&ds, myitoa((long)xchat_list_time(ph, list, "seen")));
+            Tcl_DStringEndSublist(&ds);
+        }
+
+        xchat_list_free(ph, list);
+
+    }
+
+    Tcl_AppendResult(irp, ds.string, NULL);
+
+    Tcl_DStringFree(&ds);
+
+    return TCL_OK;
+}
+
 static int tcl_chats(ClientData cd, Tcl_Interp * irp, int argc, const char *argv[])
 {
     xchat_list *list;
@@ -1739,83 +1802,26 @@ static int tcl_dcclist(ClientData cd, Tcl_Interp * irp, int argc, const char *ar
     return TCL_OK;
 }
 
-static int tcl_away(ClientData cd, Tcl_Interp * irp, int argc, const char *argv[])
+
+static int tcl_strip(ClientData cd, Tcl_Interp * irp, int argc, const char *argv[])
 {
-    xchat_context *origctx, *ctx;
-    BADARGS(1, 2, " ?server|network|context?");
+    char *new_text;
+    int flags = 1 | 2;
 
-    origctx = xchat_get_context(ph);
+    BADARGS(2, 3, " text ?flags?");
 
-    if (argc == 2) {
-        ctx = xchat_smart_context(argv[1], NULL);
-        CHECKCTX(ctx);
-        xchat_set_context(ph, ctx);
+    if (argc == 3) {
+        if (Tcl_GetInt(irp, argv[2], &flags) != TCL_OK)
+            return TCL_ERROR;
     }
 
-    Tcl_AppendResult(irp, xchat_get_info(ph, "away"), NULL);
+    new_text = xchat_strip(ph, argv[1], -1, flags);
 
-    xchat_set_context(ph, origctx);
-
-    return TCL_OK;
-}
-
-static int tcl_host(ClientData cd, Tcl_Interp * irp, int argc, const char *argv[])
-{
-    xchat_context *origctx, *ctx;
-    BADARGS(1, 2, " ?server|network|context?");
-
-    origctx = xchat_get_context(ph);
-
-    if (argc == 2) {
-        ctx = xchat_smart_context(argv[1], NULL);
-        CHECKCTX(ctx);
-        xchat_set_context(ph, ctx);
+    if(new_text) {
+        Tcl_AppendResult(irp, new_text, NULL);
+        xchat_free(ph, new_text);
     }
 
-    Tcl_AppendResult(irp, xchat_get_info(ph, "host"), NULL);
-
-    xchat_set_context(ph, origctx);
-
-    return TCL_OK;
-}
-
-static int tcl_network(ClientData cd, Tcl_Interp * irp, int argc, const char *argv[])
-{
-    xchat_context *origctx, *ctx;
-    BADARGS(1, 2, " ?server|network|context?");
-
-    origctx = xchat_get_context(ph);
-
-    if (argc == 2) {
-        ctx = xchat_smart_context(argv[1], NULL);
-        CHECKCTX(ctx);
-        xchat_set_context(ph, ctx);
-    }
-
-    Tcl_AppendResult(irp, xchat_get_info(ph, "network"), NULL);
-
-    xchat_set_context(ph, origctx);
-
-    return TCL_OK;
-}
-
-static int tcl_me(ClientData cd, Tcl_Interp * irp, int argc, const char *argv[])
-{
-    xchat_context *origctx, *ctx;
-    BADARGS(1, 2, " ?server|network|context?");
-
-    origctx = xchat_get_context(ph);
-
-    if (argc == 2) {
-        ctx = xchat_smart_context(argv[1], NULL);
-        CHECKCTX(ctx);
-        xchat_set_context(ph, ctx);
-    }
-
-    Tcl_AppendResult(irp, xchat_get_info(ph, "nick"), NULL);
-
-    xchat_set_context(ph, origctx);
-
     return TCL_OK;
 }
 
@@ -1862,7 +1868,6 @@ static int Command_Alias(char *word[], char *word_eol[], void *userdata)
     xchat_context *origctx;
     int dummy;
     char *string;
-    const char *errorInfo;
 
     if (complete_level == MAX_COMPLETES)
         return XCHAT_EAT_NONE;
@@ -1879,8 +1884,8 @@ static int Command_Alias(char *word[], char *word_eol[], void *userdata)
         aliasPtr = Tcl_GetHashValue(entry);
         origctx = xchat_get_context(ph);
         if (EvalInternalProc(aliasPtr->procPtr, 2, string, word_eol[2]) == TCL_ERROR) {
-            errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
-            xchat_printf(ph, "\0039Tcl plugin\003\tERROR (alias %s) %s \n", string, errorInfo);
+            xchat_printf(ph, "\0039Tcl plugin\003\tERROR (alias %s) ", string);
+            NiceErrorInfo ();
         }
         xchat_set_context(ph, origctx);
     }
@@ -1899,7 +1904,6 @@ static int Null_Command_Alias(char *word[], char *word_eol[], void *userdata)
     const char *channel;
     char *string;
     Tcl_DString ds;
-    const char *errorInfo;
     static int recurse = 0;
 
     if (recurse)
@@ -1927,8 +1931,8 @@ static int Null_Command_Alias(char *word[], char *word_eol[], void *userdata)
         aliasPtr = Tcl_GetHashValue(entry);
         origctx = xchat_get_context(ph);
         if (EvalInternalProc(aliasPtr->procPtr, 2, string, word_eol[1]) == TCL_ERROR) {
-            errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
-            xchat_printf(ph, "\0039Tcl plugin\003\tERROR (alias %s) %s \n", string, errorInfo);
+            xchat_printf(ph, "\0039Tcl plugin\003\tERROR (alias %s) ", string);
+            NiceErrorInfo ();
         }
         xchat_set_context(ph, origctx);
     }
@@ -1945,9 +1949,9 @@ static int Command_TCL(char *word[], char *word_eol[], void *userdata)
     const char *errorInfo;
     if (Tcl_Eval(interp, word_eol[2]) == TCL_ERROR) {
         errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
-        xchat_printf(ph, "\0039Tcl plugin\003\tERROR: %s\n", errorInfo);
+        xchat_printf(ph, "\0039Tcl plugin\003\tERROR: %s ", errorInfo);
     } else
-        xchat_printf(ph, "\0039Tcl plugin\003\tRESULT: %s\n", Tcl_GetStringResult(interp));
+        xchat_printf(ph, "\0039Tcl plugin\003\tRESULT: %s ", Tcl_GetStringResult(interp));
     return XCHAT_EAT_ALL;
 }
 
@@ -1982,7 +1986,7 @@ static int Command_Source(char *word[], char *word_eol[], void *userdata)
 
         if (Tcl_EvalFile(interp, ds.string) == TCL_ERROR) {
             errorInfo = Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY);
-            xchat_printf(ph, "\0039Tcl plugin\003\tERROR: %s\n", errorInfo);
+            xchat_printf(ph, "\0039Tcl plugin\003\tERROR: %s ", errorInfo);
         } else
             xchat_printf(ph, "\0039Tcl plugin\003\tSourced %s\n", ds.string);
 
@@ -2024,22 +2028,19 @@ static void Tcl_Plugin_Init()
     nextprocid = 0x1000;
 
     Tcl_CreateCommand(interp, "alias", tcl_alias, NULL, NULL);
-    Tcl_CreateCommand(interp, "away", tcl_away, NULL, NULL);
-    Tcl_CreateCommand(interp, "channel", tcl_channel, NULL, NULL);
     Tcl_CreateCommand(interp, "channels", tcl_channels, NULL, NULL);
     Tcl_CreateCommand(interp, "chats", tcl_chats, NULL, NULL);
     Tcl_CreateCommand(interp, "command", tcl_command, NULL, NULL);
     Tcl_CreateCommand(interp, "complete", tcl_complete, NULL, NULL);
     Tcl_CreateCommand(interp, "dcclist", tcl_dcclist, NULL, NULL);
+    Tcl_CreateCommand(interp, "notifylist", tcl_notifylist, NULL, NULL);
     Tcl_CreateCommand(interp, "findcontext", tcl_findcontext, NULL, NULL);
     Tcl_CreateCommand(interp, "getcontext", tcl_getcontext, NULL, NULL);
     Tcl_CreateCommand(interp, "getinfo", tcl_getinfo, NULL, NULL);
     Tcl_CreateCommand(interp, "getlist", tcl_getlist, NULL, NULL);
-    Tcl_CreateCommand(interp, "host", tcl_host, NULL, NULL);
     Tcl_CreateCommand(interp, "ignores", tcl_ignores, NULL, NULL);
     Tcl_CreateCommand(interp, "killtimer", tcl_killtimer, NULL, NULL);
     Tcl_CreateCommand(interp, "me", tcl_me, NULL, NULL);
-    Tcl_CreateCommand(interp, "network", tcl_network, NULL, NULL);
     Tcl_CreateCommand(interp, "on", tcl_on, NULL, NULL);
     Tcl_CreateCommand(interp, "off", tcl_off, NULL, NULL);
     Tcl_CreateCommand(interp, "nickcmp", tcl_xchat_nickcmp, NULL, NULL);
@@ -2048,17 +2049,14 @@ static void Tcl_Plugin_Init()
     Tcl_CreateCommand(interp, "::puts", tcl_xchat_puts, NULL, NULL);
     Tcl_CreateCommand(interp, "queries", tcl_queries, NULL, NULL);
     Tcl_CreateCommand(interp, "raw", tcl_raw, NULL, NULL);
-    Tcl_CreateCommand(interp, "server", tcl_server, NULL, NULL);
     Tcl_CreateCommand(interp, "servers", tcl_servers, NULL, NULL);
     Tcl_CreateCommand(interp, "setcontext", tcl_setcontext, NULL, NULL);
-    Tcl_CreateCommand(interp, "tab", tcl_channel, NULL, NULL);
+    Tcl_CreateCommand(interp, "strip", tcl_strip, NULL, NULL);
     Tcl_CreateCommand(interp, "timer", tcl_timer, NULL, NULL);
     Tcl_CreateCommand(interp, "timerexists", tcl_timerexists, NULL, NULL);
     Tcl_CreateCommand(interp, "timers", tcl_timers, NULL, NULL);
     Tcl_CreateCommand(interp, "topic", tcl_topic, NULL, NULL);
     Tcl_CreateCommand(interp, "users", tcl_users, NULL, NULL);
-    Tcl_CreateCommand(interp, "version", tcl_version, NULL, NULL);
-    Tcl_CreateCommand(interp, "xchatdir", tcl_xchatdir, NULL, NULL);
 
     Tcl_InitHashTable(&cmdTablePtr, TCL_STRING_KEYS);
     Tcl_InitHashTable(&aliasTablePtr, TCL_STRING_KEYS);
@@ -2076,13 +2074,14 @@ static void Tcl_Plugin_Init()
         xchat_printf(ph, "Error sourcing internal 'unknown' (%s)\n", Tcl_GetStringResult(interp));
     }
 
+    if (Tcl_Eval(interp, inlinetcl) == TCL_ERROR) {
+        xchat_printf(ph, "Error sourcing internal 'inlinetcl' (%s)\n", Tcl_GetStringResult(interp));
+    }
+
     if (Tcl_Eval(interp, sourcedirs) == TCL_ERROR) {
         xchat_printf(ph, "Error sourcing internal 'sourcedirs' (%s)\n", Tcl_GetStringResult(interp));
     }
 
-    if (Tcl_Eval(interp, inlinetcl) == TCL_ERROR) {
-        xchat_printf(ph, "Error sourcing internal 'inlinetcl' (%s)\n", Tcl_GetStringResult(interp));
-    }
 }
 
 static void Tcl_Plugin_DeInit()
@@ -2138,7 +2137,7 @@ static void Tcl_Plugin_DeInit()
 
 static void banner()
 {
-    xchat_printf(ph, "Tcl plugin for XChat - Version %s\n", VERSION);
+    xchat_printf(ph, "Tcl plugin for XChat - Version %s\n", PVERSION);
     xchat_print(ph, "Copyright 2002-2005 Daniel P. Stasinski\n");
     xchat_print(ph, "http://www.scriptkitties.com/tclplugin/\n");
 }
@@ -2149,6 +2148,8 @@ int xchat_plugin_init(xchat_plugin * plugin_handle, char **plugin_name, char **p
     HINSTANCE lib;
 #endif
 
+    strncpy(PVERSION, &RCSID[19], 5);
+
     ph = plugin_handle;
 
 #ifdef WIN32
@@ -2170,14 +2171,14 @@ int xchat_plugin_init(xchat_plugin * plugin_handle, char **plugin_name, char **p
 
     *plugin_name = "Tcl";
     *plugin_desc = "Tcl scripting interface";
-    *plugin_version = VERSION;
+    *plugin_version = PVERSION;
 
     Tcl_Plugin_Init();
 
     raw_line_hook = xchat_hook_server(ph, "RAW LINE", XCHAT_PRI_NORM, Server_raw_line, NULL);
     Command_TCL_hook = xchat_hook_command(ph, "tcl", XCHAT_PRI_NORM, Command_TCL, 0, 0);
     Command_Source_hook = xchat_hook_command(ph, "source", XCHAT_PRI_NORM, Command_Source, 0, 0);
-    Command_Reload_hook = xchat_hook_command(ph, "Reload", XCHAT_PRI_NORM, Command_Reload, 0, 0);
+    Command_Reload_hook = xchat_hook_command(ph, "reload", XCHAT_PRI_NORM, Command_Reload, 0, 0);
     Command_Load_hook = xchat_hook_command(ph, "LOAD", XCHAT_PRI_NORM, Command_Source, 0, 0);
     Event_Handler_hook = xchat_hook_timer(ph, 100, TCL_Event_Handler, 0);
     Null_Command_hook = xchat_hook_command(ph, "", XCHAT_PRI_NORM, Null_Command_Alias, "", 0);
@@ -2188,7 +2189,7 @@ int xchat_plugin_init(xchat_plugin * plugin_handle, char **plugin_name, char **p
     return 1;                   /* return 1 for success */
 }
 
-int xchat_plugin_deinit()
+int xchat_plugin_deinit(void)
 {
     if (reinit_tried) {
         reinit_tried--;
@@ -2213,9 +2214,11 @@ int xchat_plugin_deinit()
 
 void xchat_plugin_get_info(char **name, char **desc, char **version, void **reserved)
 {
+   strncpy(PVERSION, &RCSID[19], 5);
    *name = "tclplugin";
    *desc = "Tcl plugin for XChat";
-   *version = VERSION;
+   *version = PVERSION;
    if (reserved)
       *reserved = NULL;
 }
+
