@@ -971,6 +971,39 @@ const char *
 xchat_get_info (xchat_plugin *ph, const char *id)
 {
 	session *sess;
+	guint32 hash = str_hash (id);
+
+	/* do the session independant ones first */
+	switch (hash)
+	{
+	case 0x325acab5:	/* libdirfs */
+		return XCHATLIBDIR;
+
+	case 0x14f51cd8: /* version */
+		return PACKAGE_VERSION;
+
+	case 0xdd9b1abd:	/* xchatdir */
+		return get_xdir_utf8 ();
+
+	case 0xe33f6c4a:	/* xchatdirfs */
+		return get_xdir_fs ();
+	}
+
+	/* do the session independant ones first */
+	switch (str_hash (id))
+	{
+	case 0x325acab5:	/* libdirfs */
+		return XCHATLIBDIR;
+
+	case 0x14f51cd8: /* version */
+		return PACKAGE_VERSION;
+
+	case 0xdd9b1abd:	/* xchatdir */
+		return get_xdir_utf8 ();
+
+	case 0xe33f6c4a:	/* xchatdirfs */
+		return get_xdir_fs ();
+	}
 
 	sess = ph->context;
 	if (!is_session (sess))
@@ -979,7 +1012,7 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 		return NULL;
 	}
 
-	switch (str_hash (id))
+	switch (hash)
 	{
 	case 0x2de2ee: /* away */
 		if (sess->server->is_away)
@@ -1007,9 +1040,6 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 	case 0x1c0e99c1: /* inputbox */
 		return fe_get_inputbox_contents (sess);
 
-	case 0x325acab5:	/* libdirfs */
-		return XCHATLIBDIR;
-
 	case 0x6de15a2e:	/* network */
 		return server_get_network (sess->server, FALSE);
 
@@ -1029,9 +1059,6 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 	case 0x696cd2f: /* topic */
 		return sess->topic;
 
-	case 0x14f51cd8: /* version */
-		return VERSION;
-
 	case 0x506d600b: /* native win_ptr */
 		return fe_gui_info_ptr (sess, 0);
 
@@ -1043,12 +1070,6 @@ xchat_get_info (xchat_plugin *ph, const char *id)
 		case 2: return "hidden";
 		}
 		return NULL;
-
-	case 0xdd9b1abd:	/* xchatdir */
-		return get_xdir_utf8 ();
-
-	case 0xe33f6c4a:	/* xchatdirfs */
-		return get_xdir_fs ();
 	}
 
 	return NULL;
@@ -1451,10 +1472,11 @@ int
 xchat_emit_print (xchat_plugin *ph, const char *event_name, ...)
 {
 	va_list args;
+	/* currently only 4 because no events use more than 4.
+		This can be easily expanded without breaking the API. */
 	char *argv[4] = {NULL, NULL, NULL, NULL};
 	int i = 0;
 
-	memset (&argv, 0, sizeof (argv));
 	va_start (args, event_name);
 	while (1)
 	{
