@@ -1115,14 +1115,11 @@ pevent_load (char *filename)
 	int penum = 0;
 	char *ofs;
 
-	buf = malloc (1000);
 	if (filename == NULL)
-		snprintf (buf, 1000, "%s/pevents.conf", get_xdir_fs ());
-   else
-      safe_strcpy (buf, filename, 1000);
+		fd = xchat_open_file ("pevents.conf", O_RDONLY, 0, 0);
+	else
+		fd = xchat_open_file (filename, O_RDONLY, 0, XOF_FULLPATH);
 
-	fd = open (buf, O_RDONLY | OFLAGS);
-	free (buf);
 	if (fd == -1)
 		return 1;
 	if (fstat (fd, &st) != 0)
@@ -1238,7 +1235,7 @@ load_text_events ()
 /*
 	CL: format_event now handles filtering of arguments:
 	1) if prefs.stripcolor is set, filter all style control codes from arguments
-	2) always strip \001 (ATTR_HIDDEN) from arguments: it is only for use in the format string itself
+	2) always strip \010 (ATTR_HIDDEN) from arguments: it is only for use in the format string itself
 */
 #define ARG_FLAG(argn) (1 << argn)
 
@@ -1583,10 +1580,11 @@ pevent_save (char *fn)
 	char buf[1024];
 
 	if (!fn)
-		snprintf (buf, sizeof (buf), "%s/pevents.conf", get_xdir_fs ());
+		fd = xchat_open_file ("pevents.conf", O_CREAT | O_TRUNC | O_WRONLY,
+									 0x180, XOF_DOMODE);
 	else
-		safe_strcpy (buf, fn, sizeof (buf));
-	fd = open (buf, O_CREAT | O_TRUNC | O_WRONLY | OFLAGS, 0x180);
+		fd = xchat_open_file (fn, O_CREAT | O_TRUNC | O_WRONLY, 0x180,
+									 XOF_FULLPATH | XOF_DOMODE);
 	if (fd == -1)
 	{
 		/*
@@ -1650,7 +1648,7 @@ sound_find_command (void)
 }
 
 void
-sound_play (const char *file)
+sound_play (const char *file, gboolean quiet)
 {
 	char buf[512];
 	char wavfile[512];
@@ -1661,9 +1659,6 @@ sound_play (const char *file)
 	if (!file[0])
 		return;
 
-	memset (buf, 0, sizeof (buf));
-	memset (wavfile, 0, sizeof (wavfile));
-
 #ifdef WIN32
 	/* check for fullpath, windows style */
 	if (strlen (file) > 3 &&
@@ -1679,6 +1674,7 @@ sound_play (const char *file)
 	{
 		strncpy (wavfile, file, sizeof (wavfile));
 	}
+	wavfile[sizeof (wavfile) - 1] = 0;	/* ensure termination */
 
 	file_fs = g_filename_from_utf8 (wavfile, -1, 0, 0, 0);
 	if (!file_fs)
@@ -1711,8 +1707,11 @@ sound_play (const char *file)
 
 	} else
 	{
-		snprintf (buf, sizeof (buf), _("Cannot read sound file:\n%s"), wavfile);
-		fe_message (buf, FE_MSG_ERROR);
+		if (!quiet)
+		{
+			snprintf (buf, sizeof (buf), _("Cannot read sound file:\n%s"), wavfile);
+			fe_message (buf, FE_MSG_ERROR);
+		}
 	}
 
 	g_free (file_fs);
@@ -1722,7 +1721,7 @@ void
 sound_play_event (int i)
 {
 	if (sound_files[i])
-		sound_play (sound_files[i]);
+		sound_play (sound_files[i], FALSE);
 }
 
 static void
@@ -1747,8 +1746,7 @@ sound_load ()
 
 	memset (&sound_files, 0, sizeof (char *) * (NUM_XP));
 
-	snprintf (buf, sizeof (buf), "%s/sound.conf", get_xdir_fs ());
-	fd = open (buf, O_RDONLY | OFLAGS);
+	fd = xchat_open_file ("sound.conf", O_RDONLY, 0, 0);
 	if (fd == -1)
 		return;
 
@@ -1778,8 +1776,8 @@ sound_save ()
 	int fd, i;
 	char buf[512];
 
-	snprintf (buf, sizeof (buf), "%s/sound.conf", get_xdir_fs ());
-	fd = open (buf, O_CREAT | O_TRUNC | O_WRONLY | OFLAGS, 0x180);
+	fd = xchat_open_file ("sound.conf", O_CREAT | O_TRUNC | O_WRONLY, 0x180,
+								 XOF_DOMODE);
 	if (fd == -1)
 		return;
 
