@@ -52,13 +52,7 @@ void *xchat_realloc (char *old, int len, char *file, int line);
 #endif
 
 /* force a 32bit CMP.L */
-#ifdef _MSC_VER
-#define CMPL(a, c0, c1, c2, c3) (*(guint32 *)(a) == (guint32)(c0 | (c1 << 8) | (c2 << 16) | (c3 << 24)))
-#else
 #define CMPL(a, c0, c1, c2, c3) (a == (guint32)(c0 | (c1 << 8) | (c2 << 16) | (c3 << 24)))
-#endif
-/* force a 16bit CMP.W */
-#define CMPW(a, c0, c1) (*(guint16 *)(a) == (guint16)(c0 | (c1 << 8)))
 #define WORDL(c0, c1, c2, c3) (guint32)(c0 | (c1 << 8) | (c2 << 16) | (c3 << 24))
 #define WORDW(c0, c1) (guint16)(c0 | (c1 << 8))
 
@@ -85,6 +79,7 @@ void *xchat_realloc (char *old, int len, char *file, int line);
 #define CHANLEN		300
 #define PDIWORDS		32
 #define USERNAMELEN 10
+#define HIDDEN_CHAR	8			/* invisible character for xtext */
 
 #define safe_strcpy(dest,src,len)	{strncpy((dest),(src),(len)); \
 												(dest)[len-1] = 0;}
@@ -147,6 +142,7 @@ struct xchatprefs
 	char proxy_host[64];
 	int proxy_port;
 	int proxy_type; /* 0=disabled, 1=wingate 2=socks4, 3=socks5, 4=http */
+	int proxy_use; /* 0=all 1=IRC_ONLY 2=DCC_ONLY */
 	unsigned int proxy_auth;
 	char proxy_user[32];
 	char proxy_pass[32];
@@ -162,6 +158,7 @@ struct xchatprefs
 	int away_size_max;
 	int paned_pos;
 	int tabs_position;
+	int tab_layout;
 	int max_auto_indent;
 	int dcc_blocksize;
 	int max_lines;
@@ -176,7 +173,10 @@ struct xchatprefs
 	int mainwindow_top;
 	int mainwindow_width;
 	int mainwindow_height;
+	int completion_sort;
 	int gui_win_state;
+	int gui_usermenu;
+	int gui_join_dialog;
 	int dialog_left;
 	int dialog_top;
 	int dialog_width;
@@ -185,18 +185,16 @@ struct xchatprefs
 	int recon_delay;
 	int bantype;
 	int userlist_sort;
-	int nu_color;
-	unsigned long local_ip;
-	unsigned long dcc_ip;
+	guint32 local_ip;
+	guint32 dcc_ip;
 	char dcc_ip_str[DOMAINLEN + 1];
 
 	unsigned int tab_small;
-	unsigned int tab_dnd;
 	unsigned int tab_sort;
+	unsigned int tab_icons;
 	unsigned int mainwindow_save;
 	unsigned int perc_color;
 	unsigned int perc_ascii;
-	unsigned int use_trans;
 	unsigned int autosave;
 	unsigned int autodialog;
 	unsigned int autosave_url;
@@ -222,14 +220,12 @@ struct xchatprefs
 	unsigned int autoopendccrecvwindow;
 	unsigned int autoopendccchatwindow;
 	unsigned int transparent;
-	unsigned int tint;
 	unsigned int stripcolor;
 	unsigned int timestamp;
 	unsigned int fastdccsend;
 	unsigned int dcc_send_fillspaces;
 	unsigned int dcc_remove;
 	unsigned int slist_skip;
-	unsigned int slist_edit;
 	unsigned int slist_select;
 	unsigned int filterbeep;
 	unsigned int beepmsg;
@@ -238,11 +234,11 @@ struct xchatprefs
 	unsigned int flash_hilight;
 	unsigned int truncchans;
 	unsigned int privmsgtab;
+	unsigned int irc_join_delay;
 	unsigned int logging;
 	unsigned int timestamp_logs;
 	unsigned int newtabstofront;
 	unsigned int dccwithnick;
-	unsigned int hilitenotify;
 	unsigned int hidever;
 	unsigned int ip_from_server;
 	unsigned int raw_modes;
@@ -263,15 +259,14 @@ struct xchatprefs
 	unsigned int thin_separator;
 	unsigned int auto_indent;
 	unsigned int wordwrap;
+	unsigned int gui_input_spell;
 	unsigned int throttle;
-	unsigned int fudgeservernotice;
 	unsigned int topicbar;
 	unsigned int hideuserlist;
 	unsigned int hidemenu;
 	unsigned int perlwarnings;
 	unsigned int lagometer;
 	unsigned int throttlemeter;
-	unsigned int hebrew;
 	unsigned int pingtimeout;
 	unsigned int whois_on_notifyonline;
 	unsigned int wait_on_exit;
@@ -349,6 +344,14 @@ typedef struct session
 	int done_away_check:1;	/* done checking for away status changes */
 } session;
 
+struct msproxy_state_t
+{
+	gint32				clientid;
+	gint32				serverid;
+	unsigned char		seq_recv;		/* seq number of last packet recv.	*/
+	unsigned char		seq_sent;		/* seq number of last packet sent.	*/
+};
+
 typedef struct server
 {
 	/*  server control operations (in server*.c) */
@@ -366,7 +369,8 @@ typedef struct server
 	void (*p_quit)(struct server *, char *reason);
 	void (*p_kick)(struct server *, char *channel, char *nick, char *reason);
 	void (*p_part)(struct server *, char *channel, char *reason);
-	void (*p_nickserv)(struct server *, char *pass);
+	void (*p_ns_identify)(struct server *, char *pass);
+	void (*p_ns_ghost)(struct server *, char *usname, char *pass);
 	void (*p_join)(struct server *, char *channel, char *key);
 	void (*p_login)(struct server *, char *user, char *realname);
 	void (*p_join_info)(struct server *, char *channel);
@@ -392,8 +396,12 @@ typedef struct server
 
 	int port;
 	int sok;					/* is equal to sok4 or sok6 (the one we are using) */
-	int sok4;				/* tcp4 socket */
-	int sok6;				/* tcp6 socket */
+	int sok4;					/* tcp4 socket */
+	int sok6;					/* tcp6 socket */
+	int proxy_sok;				/* Additional information for MS Proxy beast */
+	int proxy_sok4;
+	int proxy_sok6;
+	struct msproxy_state_t msp_state;
 	int id;					/* unique ID number (for plugin API) */
 #ifdef USE_OPENSSL
 	SSL *ssl;
@@ -404,11 +412,12 @@ typedef struct server
 	int childpid;
 	int iotag;
 	int recondelay_tag;				/* reconnect delay timeout */
+	int joindelay_tag;				/* waiting before we send JOIN */
 	char hostname[128];				/* real ip number */
 	char servername[128];			/* what the server says is its name */
 	char password[86];
 	char nick[NICKLEN];
-	char linebuf[522];				/* RFC says 512 including \r\n */
+	char linebuf[2048];				/* RFC says 512 chars including \r\n */
 	char *last_away_reason;
 	int pos;								/* current position in linebuf */
 	int nickcount;
@@ -502,4 +511,7 @@ struct popup
 	char *name;
 };
 
+/* CL: get a random int in the range [0..n-1]. DON'T use rand() % n, it gives terrible results. */
+#define RAND_INT(n) ((int)(rand() / (RAND_MAX + 1.0) * (n)))
+
 #endif
