@@ -25,6 +25,9 @@
 #include <gconf/gconf-client.h>
 #include <libgnomeui/gnome-stock-icons.h>
 #include <libgnomevfs/gnome-vfs.h>
+#ifdef HAVE_LIBSEXY
+#include <libsexy/sexy-tooltip.h>
+#endif
 
 #include "conversation-panel.h"
 #include "gui.h"
@@ -37,6 +40,7 @@
 #include "../common/outbound.h"
 #include "../common/text.h"
 #include "../common/url.h"
+#include "../common/userlist.h"
 #include "../common/util.h"
 #include "../common/xchatc.h"
 
@@ -53,6 +57,17 @@ static void     conversation_panel_clicked_word       (GtkWidget              *x
                                                        char                   *word,
                                                        GdkEventButton         *event,
                                                        ConversationPanel      *panel);
+#ifdef HAVE_LIBSEXY
+static void     conversation_panel_enter_word         (GtkWidget              *xtext,
+                                                       char                   *word,
+                                                       ConversationPanel      *panel);
+static void     conversation_panel_leave_word         (GtkWidget              *xtext,
+                                                       char                   *word,
+                                                       ConversationPanel      *panel);
+static gboolean conversation_panel_lost_focus         (GtkWidget              *widget,
+                                                       GdkEventFocus          *event,
+                                                       ConversationPanel      *panel);
+#endif
 static void     conversation_panel_set_font           (ConversationPanel      *panel);
 static void     conversation_panel_font_changed       (GConfClient            *client,
                                                        guint                   cnxn_id,
@@ -67,7 +82,12 @@ static void     timestamps_changed                    (GConfClient            *c
                                                        guint                   cnxn_id,
                                                        GConfEntry             *entry,
                                                        xtext_buffer           *buffer);
-static void     conversation_panel_print_line         (xtext_buffer           *buffer,
+static void     redundant_nickstamp_changed           (GConfClient            *client,
+                                                       guint                   cnxn_id,
+                                                       GConfEntry             *entry,
+                                                       ConversationPanel      *panel);
+static void     conversation_panel_print_line         (ConversationPanel      *panel,
+                                                       xtext_buffer           *buffer,
                                                        guchar                  *text,
                                                        int                     len,
                                                        gboolean                indent);
@@ -89,6 +109,9 @@ static void     drop_paste_filename                   (GtkAction              *a
                                                        ConversationPanel      *panel);
 static void     drop_cancel                           (GtkAction              *action,
                                                        ConversationPanel      *panel);
+static void     style_set_callback                    (GtkWidget              *widget,
+                                                       GtkStyle               *previous_style,
+                                                       void                   *data);
 static void     drag_data_received                    (GtkWidget              *widget,
                                                        GdkDragContext         *context,
                                                        gint                    x,
@@ -104,8 +127,17 @@ static void     send_file                             (gpointer                f
                                                        gpointer                user_data);
 static void     on_default_copy_activate              (GtkAction              *action,
                                                        ConversationPanel      *panel);
-static gboolean uri_is_text                           (gchar *uri);
-static gboolean check_file_size                       (gchar *uri);
+static gboolean uri_is_text                           (gchar                  *uri);
+static gboolean check_file_size                       (gchar                  *uri);
+#ifdef HAVE_LIBSEXY
+GtkWidget* get_user_vbox_infos                        (struct User            *user);
+#endif
+static void redraw_transparency                       (ConversationPanel      *panel);
+#ifdef USE_XLIB
+static GdkFilterReturn root_event_cb                  (GdkXEvent              *xev,
+                                                       GdkEventProperty       *event,
+                                                       ConversationPanel      *panel);
+#endif
 
 struct _fe_lastlog_info
 {
@@ -126,6 +158,12 @@ struct _ConversationPanelPriv
 
 	gchar          *selected_word;
 	GSList         *dropped_files;
+
+#ifdef HAVE_LIBSEXY
+	GtkWidget      *current_tooltip;
+	guint           tooltip_timeout;
+#endif
+	gboolean        redundant_nickstamps;
 };
 
 static GtkActionEntry url_actions[] = {
@@ -169,6 +207,8 @@ static GtkTargetEntry target_table[] = {
 	{ "text/plain",     0, TARGET_TEXT_PLAIN }
 };
 
+static struct User dialog_user;
+
 static GtkHBoxClass *parent_class;
 G_DEFINE_TYPE (ConversationPanel, conversation_panel, GTK_TYPE_HBOX);
 
@@ -192,6 +232,7 @@ conversation_panel_init (ConversationPanel *panel)
 {
 	GtkWidget *frame;
 	GtkActionGroup *action_group;
+	GConfClient       *client;
 
 	panel->priv = g_new0 (ConversationPanelPriv, 1);
 	panel->priv->xtext     = gtk_xtext_new (colors, TRUE);
@@ -222,9 +263,26 @@ conversation_panel_init (ConversationPanel *panel)
 	gtk_ui_manager_insert_action_group (gui.manager, action_group, 0);
 	g_object_unref (action_group);
 
+#ifdef HAVE_LIBSEXY
+	panel->priv->tooltip_timeout = 0;
+	panel->priv->current_tooltip = NULL;
+#endif
+	client = gconf_client_get_default ();
+	panel->priv->redundant_nickstamps =
+		gconf_client_get_bool (client, "/apps/xchat/main_window/redundant_nickstamps", NULL);
+	g_object_unref (client);
+
+	g_signal_connect (G_OBJECT (panel), "style_set", G_CALLBACK (style_set_callback), panel);
 	g_signal_connect (G_OBJECT (panel->priv->xtext), "drag_data_received", G_CALLBACK (drag_data_received), panel);
 	gtk_drag_dest_set (panel->priv->xtext, GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP,
 	                   target_table, G_N_ELEMENTS (target_table), GDK_ACTION_COPY | GDK_ACTION_ASK);
+
+#ifdef USE_XLIB
+	gdk_window_set_events (gdk_get_default_root_window (), GDK_PROPERTY_CHANGE_MASK);
+	gdk_window_add_filter (gdk_get_default_root_window (), (GdkFilterFunc)root_event_cb, panel);
+#endif
+
+	conversation_panel_set_show_marker (panel, prefs.show_marker);
 }
 
 static void
@@ -267,7 +325,12 @@ conversation_panel_realize (GtkWidget *widget)
 	conversation_panel_set_font       (panel);
 	conversation_panel_set_background (panel);
 
-	g_signal_connect (G_OBJECT (panel->priv->xtext), "word_click", G_CALLBACK (conversation_panel_clicked_word), panel);
+	g_signal_connect (G_OBJECT (panel->priv->xtext), "word_click",      G_CALLBACK (conversation_panel_clicked_word), panel);
+#ifdef HAVE_LIBSEXY
+	g_signal_connect (G_OBJECT (panel->priv->xtext), "word_enter",      G_CALLBACK (conversation_panel_enter_word),   panel);
+	g_signal_connect (G_OBJECT (panel->priv->xtext), "word_leave",      G_CALLBACK (conversation_panel_leave_word),   panel);
+	g_signal_connect (G_OBJECT (gui.main_window),    "focus-out-event", G_CALLBACK (conversation_panel_lost_focus),   panel);
+#endif
 	gconf_client_notify_add (client, "/apps/xchat/main_window/use_sys_fonts",
 	                         (GConfClientNotifyFunc) conversation_panel_font_changed,       panel, NULL, NULL);
 	gconf_client_notify_add (client, "/apps/xchat/main_window/font",
@@ -278,6 +341,9 @@ conversation_panel_realize (GtkWidget *widget)
 	                         (GConfClientNotifyFunc) conversation_panel_background_changed, panel, NULL, NULL);
 	gconf_client_notify_add (client, "/apps/xchat/main_window/background_transparency",
 	                         (GConfClientNotifyFunc) conversation_panel_background_changed, panel, NULL, NULL);
+	gconf_client_notify_add (client, "/apps/xchat/main_window/redundant_nickstamps",
+	                         (GConfClientNotifyFunc) redundant_nickstamp_changed, panel, NULL, NULL);
+
 
 	g_object_unref (client);
 }
@@ -289,11 +355,16 @@ conversation_panel_check_word (GtkWidget *xtext, char *word, int len)
         current_sess = gui.current_session;
 
 	url = url_check_word (word, len);
-	if (url == 0) {
-		if (((word[0]=='@' || word[0]=='+') && userlist_find (current_sess, word+1)) || userlist_find (current_sess, word))
-			return WORD_NICK;
-		if (current_sess->type == SESS_DIALOG)
+	if (url == 0 && current_sess) {
+		if (current_sess->type == SESS_DIALOG) {
+			if (strcmp (word, current_sess->channel) == 0)
+				return WORD_NICK;
 			return WORD_DIALOG;
+		} else if (((word[0]=='@' || word[0]=='+') && userlist_find (current_sess, word+1)) ||
+			   userlist_find (current_sess, word)) {
+			if (strcmp (word, current_sess->server->nick) != 0)
+				return WORD_NICK;
+		}
 	}
 	return url;
 }
@@ -320,6 +391,10 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 				char *command;
 				struct session *sess;
 
+				if ((gui.current_session->type == SESS_DIALOG) &&
+				    (strcmp (gui.current_session->channel, word) == 0))
+					break;
+
 				sess = find_dialog (gui.current_session->server, word);
 				if (sess)
 					navigation_tree_select_session (gui.server_tree, sess);
@@ -354,7 +429,17 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 				GtkWidget *menu;
 				menu = gtk_ui_manager_get_widget (gui.manager, "/DefaultPopup");
 				/* FIXME: we should not display the copy action if no text is selected */
-				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time ());
+				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, event->time);
+			}
+			break;
+		case WORD_DIALOG:
+			{
+				GtkWidget *menu;
+				menu = gtk_ui_manager_get_widget (gui.manager, "/DialogPopup");
+				strcpy (dialog_user.nick, gui.current_session->channel);
+				dialog_user.nick[strlen (gui.current_session->channel)] = '\0';
+				current_user = &dialog_user;
+				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, event->time);
 			}
 			break;
 		case WORD_URL:
@@ -365,7 +450,7 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 				if (panel->priv->selected_word)
 					g_free (panel->priv->selected_word);
 				panel->priv->selected_word = g_strdup (word);
-				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time ());
+				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, event->time);
 			}
 			break;
 		case WORD_NICK:
@@ -373,15 +458,23 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 				struct User *user;
 				GtkWidget   *menu;
 
-				menu = gtk_ui_manager_get_widget (gui.manager, "/UserlistPopup");
 				if (panel->priv->selected_word)
 					g_free (panel->priv->selected_word);
 				panel->priv->selected_word = g_strdup (word);
 
-				user = userlist_find (gui.current_session, word);
-				if (user) {
-					current_user = user;
-					gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time ());
+				if (gui.current_session->type == SESS_CHANNEL) {
+					menu = gtk_ui_manager_get_widget (gui.manager, "/UserlistPopup");
+					user = userlist_find (gui.current_session, word);
+					if (user) {
+						current_user = user;
+						gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, event->time);
+					}
+				} else if (gui.current_session->type == SESS_DIALOG) {
+					menu = gtk_ui_manager_get_widget (gui.manager, "/UserDialogPopup");
+					strcpy (dialog_user.nick, word);
+					dialog_user.nick[strlen (word)] = '\0';
+					current_user = &dialog_user;
+					gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, event->time);
 				}
 			}
 			break;
@@ -395,16 +488,96 @@ conversation_panel_clicked_word (GtkWidget *xtext, char *word, GdkEventButton *e
 				if (panel->priv->selected_word)
 					g_free (panel->priv->selected_word);
 				panel->priv->selected_word = g_strdup (word);
-				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time ());
+				gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, event->time);
 			}
 			break;
-		case WORD_DIALOG:
-			/* FIXME: show dialog context menu */
-			break;
 		}
 	}
 }
 
+#ifdef HAVE_LIBSEXY
+static gboolean
+show_user_tooltip (gchar *nick)
+{
+	GtkWidget *tooltip;
+	ConversationPanel *panel;
+	struct User *user = NULL;
+	GdkRectangle rect;
+	GdkScreen *screen;
+	GtkWidget *vbox;
+	GdkDisplay *display;
+
+	panel = CONVERSATION_PANEL(gui.conversation_panel);
+
+	user = userlist_find (gui.current_session, nick);
+
+	if (user) {
+		tooltip = sexy_tooltip_new ();
+
+		display = gdk_display_get_default ();
+		gdk_display_get_pointer (display, &screen, &rect.x, &rect.y, NULL);
+		rect.y += 10;
+		rect.width = 1;
+		rect.height = 1;
+
+		vbox = get_user_vbox_infos (user);
+		gtk_container_add (GTK_CONTAINER (tooltip), vbox);
+		sexy_tooltip_position_to_rect (SEXY_TOOLTIP (tooltip), &rect, screen);
+		gtk_widget_show (tooltip);
+
+		panel->priv->current_tooltip = tooltip;
+	}
+
+	return FALSE;
+}
+
+static void
+conversation_panel_enter_word (GtkWidget *xtext, char *word, ConversationPanel *panel)
+{
+	switch (conversation_panel_check_word (xtext, word, strlen (word))) {
+		case WORD_NICK:
+			{
+				panel->priv->tooltip_timeout =
+					g_timeout_add (500, (GSourceFunc) show_user_tooltip, word);
+				break;
+			}
+	}
+}
+
+static void
+conversation_panel_remove_tooltip (ConversationPanel *panel)
+{
+	if (panel->priv->tooltip_timeout) {
+		g_source_remove (panel->priv->tooltip_timeout);
+		panel->priv->tooltip_timeout = 0;
+	}
+
+	if (panel->priv->current_tooltip) {
+		gtk_widget_destroy (panel->priv->current_tooltip);
+		panel->priv->current_tooltip = NULL;
+	}
+}
+
+static void
+conversation_panel_leave_word (GtkWidget *xtext, char *word, ConversationPanel *panel)
+{
+	switch (conversation_panel_check_word (xtext, word, strlen (word))) {
+		case WORD_NICK:
+			{
+				conversation_panel_remove_tooltip (panel);
+				break;
+			}
+	}
+}
+
+static gboolean
+conversation_panel_lost_focus (GtkWidget *widget, GdkEventFocus *event, ConversationPanel *panel)
+{
+	conversation_panel_remove_tooltip (panel);
+	return FALSE;
+}
+#endif
+
 static void
 conversation_panel_set_font (ConversationPanel *panel)
 {
@@ -498,6 +671,12 @@ timestamps_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, xtext
 	gtk_xtext_set_time_stamp (buffer, gconf_client_get_bool (client, entry->key, NULL));
 }
 
+static void
+redundant_nickstamp_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, ConversationPanel *panel)
+{
+	panel->priv->redundant_nickstamps = gconf_value_get_bool (entry->value);
+}
+
 static void
 open_url (GtkAction *action, ConversationPanel *panel)
 {
@@ -510,8 +689,8 @@ open_url (GtkAction *action, ConversationPanel *panel)
 static void
 copy_text (GtkAction *action, ConversationPanel *panel)
 {
-	GtkClipboard *clipboard = gtk_clipboard_get (GDK_NONE);
-        gtk_clipboard_set_text (clipboard, panel->priv->selected_word, strlen (panel->priv->selected_word));
+	GtkClipboard *clipboard = gtk_widget_get_clipboard (GTK_WIDGET (panel), GDK_SELECTION_CLIPBOARD);
+        gtk_clipboard_set_text (clipboard, panel->priv->selected_word, -1);
 }
 
 static void
@@ -585,8 +764,9 @@ drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 		guchar *txt;
 
 		txt = gtk_selection_data_get_text (selection_data);
-		if (gui.current_session != NULL)
-		handle_multiline (gui.current_session, txt, TRUE, FALSE);
+		if (gui.current_session != NULL) {
+			handle_multiline (gui.current_session, txt, TRUE, FALSE);
+		}
 
 		g_free (txt);
 		break;
@@ -598,8 +778,9 @@ drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 		gint nb_uri;
 
 		if ((panel->priv->current->type != SESS_CHANNEL) &&
-		    (panel->priv->current->type != SESS_DIALOG))
+		    (panel->priv->current->type != SESS_DIALOG)) {
 			return;
+		}
 
 		if (selection_data->format != 8 || selection_data->length == 0) {
 			g_printerr (_("URI list dropped on xchat-gnome had wrong format (%d) or length (%d)\n"),
@@ -613,8 +794,9 @@ drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 
 		free_dropped_files (panel);
 
-		for (nb_uri = 0; uris[nb_uri] && strlen (uris[nb_uri]) > 0; nb_uri++)
+		for (nb_uri = 0; uris[nb_uri] && strlen (uris[nb_uri]) > 0; nb_uri++) {
 			panel->priv->dropped_files = g_slist_prepend (panel->priv->dropped_files, uris[nb_uri]);
+		}
 		g_free (uris); /* String in uris will be freed in free_dropped_files */
 		panel->priv->dropped_files = g_slist_reverse (panel->priv->dropped_files);
 
@@ -626,17 +808,19 @@ drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 			entry = gtk_ui_manager_get_widget (gui.manager, "/DropFilePopup/DropPasteFile");
 			if (nb_uri > 1 ||
 			    (uri_is_text (panel->priv->dropped_files->data) == FALSE) ||
-			    (check_file_size (panel->priv->dropped_files->data) == FALSE))
+			    (check_file_size (panel->priv->dropped_files->data) == FALSE)) {
 				gtk_widget_set_sensitive (entry, FALSE);
-			else
+			} else {
 				gtk_widget_set_sensitive (entry, TRUE);
+			}
 
 			/* Enable/Disable send files */
 			entry = gtk_ui_manager_get_widget (gui.manager, "/DropFilePopup/DropSendFiles");
-			if (panel->priv->current->type == SESS_CHANNEL)
+			if (panel->priv->current->type == SESS_CHANNEL) {
 				gtk_widget_set_sensitive (entry, FALSE);
-			else
+			} else {
 				gtk_widget_set_sensitive (entry, TRUE);
+			}
 
 			gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 2, gtk_get_current_event_time ());
 		} else {
@@ -645,16 +829,18 @@ drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 				/* Dropped in a channel */
 				if (nb_uri == 1 &&
 				    uri_is_text (panel->priv->dropped_files->data) &&
-				    check_file_size (panel->priv->dropped_files->data))
+				    check_file_size (panel->priv->dropped_files->data)) {
 					drop_paste (panel);
+				}
 			} else {
 				/* Dropped in a query */
 				if (nb_uri == 1 &&
 				    uri_is_text (panel->priv->dropped_files->data) &&
-				    check_file_size (panel->priv->dropped_files->data))
+				    check_file_size (panel->priv->dropped_files->data)) {
 					drop_paste (panel);
-				else
+				} else {
 					drop_send (panel);
+				}
 			}
 		}
 	}
@@ -676,7 +862,7 @@ drop_send (ConversationPanel *panel)
 {
 	g_return_if_fail (panel->priv->dropped_files != NULL);
 	g_slist_foreach (panel->priv->dropped_files, send_file, NULL);
-        free_dropped_files (panel);
+	free_dropped_files (panel);
 }
 
 static void
@@ -690,8 +876,9 @@ drop_paste (ConversationPanel *panel)
 	uri = panel->priv->dropped_files->data;
 	result = gnome_vfs_read_entire_file (uri, NULL, &contents);
 	if (result == GNOME_VFS_OK) {
-		if (panel->priv->current != NULL)
+		if (panel->priv->current != NULL) {
 			handle_multiline (panel->priv->current, (char *) contents, TRUE, FALSE);
+		}
 		g_free (contents);
 	} else {
 		g_printerr (_("Error reading file \"%s\": %s\n"), uri, gnome_vfs_result_to_string (result));
@@ -720,7 +907,7 @@ send_file (gpointer file, gpointer user_data)
 static void
 on_default_copy_activate (GtkAction *action, ConversationPanel *panel)
 {
-	gtk_editable_copy_clipboard (GTK_EDITABLE (gui.text_entry));
+	conversation_panel_copy_selection (panel);
 }
 
 static gboolean
@@ -759,6 +946,21 @@ check_file_size (gchar *uri)
 	return file_size_ok;
 }
 
+static void
+style_set_callback (GtkWidget *widget, GtkStyle  *previous_style, void *data)
+{
+	GConfClient *client;
+	gint color_scheme;
+	client = gconf_client_get_default ();
+	color_scheme = gconf_client_get_int (client, "/apps/xchat/irc/color_scheme", NULL);
+	g_object_unref (client);
+	if (color_scheme == 0) {
+		load_colors (color_scheme);
+		load_palette (color_scheme);
+	}
+	conversation_panel_update_colors (CONVERSATION_PANEL (gui.conversation_panel));
+}
+
 GtkWidget *
 conversation_panel_new (void)
 {
@@ -774,7 +976,7 @@ conversation_panel_update_colors (ConversationPanel *panel)
 }
 
 void
-conversation_panel_add_session (ConversationPanel *panel, struct session *sess)
+conversation_panel_add_session (ConversationPanel *panel, struct session *sess, gboolean focus)
 {
 	GConfClient  *client;
 	xtext_buffer *buffer;
@@ -791,7 +993,9 @@ conversation_panel_add_session (ConversationPanel *panel, struct session *sess)
 	g_hash_table_insert (panel->priv->buffers,            sess, buffer);
 	g_hash_table_insert (panel->priv->timestamp_notifies, sess, GINT_TO_POINTER (notify));
 
-	conversation_panel_set_current (panel, sess);
+	if (focus) {
+		conversation_panel_set_current (panel, sess);
+	}
 }
 
 void
@@ -802,6 +1006,10 @@ conversation_panel_set_current (ConversationPanel *panel, struct session *sess)
 	panel->priv->current = sess;
 	buffer = g_hash_table_lookup (panel->priv->buffers, sess);
 	gtk_xtext_buffer_show (GTK_XTEXT (panel->priv->xtext), buffer, TRUE);
+
+#ifdef HAVE_LIBSEXY
+	conversation_panel_remove_tooltip (panel);
+#endif
 }
 
 void
@@ -813,6 +1021,9 @@ conversation_panel_save_current (ConversationPanel *panel)
 	struct tm  date;
 	time_t     dtime;
 
+	if (!panel->priv->current)
+		return;
+
 	file_chooser = gtk_file_chooser_dialog_new (_("Save Transcript"),
 	                                            GTK_WINDOW (gui.main_window),
 	                                            GTK_FILE_CHOOSER_ACTION_SAVE,
@@ -871,13 +1082,14 @@ conversation_panel_clear (ConversationPanel *panel, struct session *sess)
 }
 
 static void
-conversation_panel_print_line (xtext_buffer *buffer, guchar *text, int len, gboolean indent)
+conversation_panel_print_line (ConversationPanel *panel, xtext_buffer *buffer, guchar *text, int len, gboolean indent)
 {
 	int            leftlen;
 	unsigned char *tab;
 
-	if (len == 0)
+	if (len == 0) {
 		len = 1;
+	}
 
 	if (indent == FALSE) {
 		int     stamp_size;
@@ -897,7 +1109,19 @@ conversation_panel_print_line (xtext_buffer *buffer, guchar *text, int len, gboo
 	tab = strchr (text, '\t');
 	if (tab && tab < (text + len)) {
 		leftlen = tab - text;
-		gtk_xtext_append_indent (buffer, text, leftlen, tab + 1, len - (leftlen + 1));
+
+		if(!panel->priv->redundant_nickstamps && strncmp (buffer->laststamp, text, leftlen) == 0)
+		{
+			text = tab+1;
+			len -= leftlen;
+			gtk_xtext_append_indent (buffer, 0, 0, text, len);
+		}
+		else
+		{
+			strncpy (buffer->laststamp, text, leftlen);
+			buffer->laststamp[leftlen]=0;
+			gtk_xtext_append_indent (buffer, text, leftlen, tab + 1, strlen (text) - leftlen - 1);
+		}
 	} else {
 		gtk_xtext_append_indent (buffer, 0, 0, text, len);
 	}
@@ -916,10 +1140,10 @@ conversation_panel_print (ConversationPanel *panel, struct session *sess, guchar
 	while (1) {
 		switch (*text) {
 		case '\0':
-			conversation_panel_print_line (buffer, last_text, len, indent);
+			conversation_panel_print_line (panel, buffer, last_text, len, indent);
 			return;
 		case '\n':
-			conversation_panel_print_line (buffer, last_text, len, indent);
+			conversation_panel_print_line (panel, buffer, last_text, len, indent);
 			text++;
 			if (*text == '\0')
 				return;
@@ -954,8 +1178,9 @@ conversation_panel_remove_session (ConversationPanel *panel, struct session *ses
 static void
 conversation_panel_lastlog_foreach (GtkXText *xtext, guchar *text, fe_lastlog_info *info)
 {
-	if (nocasestrstr (text, info->sstr))
+	if (nocasestrstr (text, info->sstr)) {
 		conversation_panel_print (info->panel, info->sess, text, prefs.indent_nicks);
+	}
 }
 
 void
@@ -1022,3 +1247,55 @@ conversation_panel_page_down (ConversationPanel *panel)
 		value = end;
 	gtk_adjustment_set_value (adj, value);
 }
+
+void
+conversation_panel_copy_selection (ConversationPanel *panel)
+{
+	GtkClipboard *clipboard = gtk_widget_get_clipboard (GTK_WIDGET (panel), GDK_SELECTION_CLIPBOARD);
+	gtk_xtext_copy_selection (GTK_XTEXT (panel->priv->xtext), clipboard);
+}
+
+void
+conversation_panel_queue_tdraw (ConversationPanel *panel)
+{
+	if (GTK_XTEXT (panel->priv->xtext)->transparent) {
+		gtk_widget_queue_draw (panel->priv->xtext);
+	}
+}
+
+static void
+redraw_transparency (ConversationPanel *panel)
+{
+	if (GTK_XTEXT (panel->priv->xtext)->transparent) {
+		gtk_xtext_refresh (GTK_XTEXT (panel->priv->xtext), 1);
+	}
+}
+
+#ifdef USE_XLIB
+static GdkFilterReturn
+root_event_cb (GdkXEvent *xev, GdkEventProperty *event, ConversationPanel *panel)
+{
+	static Atom at = None;
+	XEvent *xevent = (XEvent *)xev;
+
+	if (xevent->type == PropertyNotify) {
+		if (at == None)
+			at = XInternAtom (xevent->xproperty.display, "_XROOTPMAP_ID", True);
+		if (at == xevent->xproperty.atom)
+			redraw_transparency (panel);
+	}
+	return GDK_FILTER_CONTINUE;
+}
+#endif
+
+void
+conversation_panel_check_marker_visibility (ConversationPanel *panel)
+{
+	gtk_xtext_check_marker_visibility (GTK_XTEXT (panel->priv->xtext));
+}
+
+void
+conversation_panel_set_show_marker (ConversationPanel *panel, gboolean show_marker)
+{
+	gtk_xtext_set_show_marker (GTK_XTEXT (panel->priv->xtext), show_marker);
+}
