@@ -63,7 +63,6 @@ static void on_edit_cut_activate (GtkAction *action, gpointer data);
 static void on_edit_copy_activate (GtkAction *action, gpointer data);
 static void on_edit_paste_activate (GtkAction *action, gpointer data);
 static void on_edit_preferences_activate (GtkAction *action, gpointer data);
-static void on_network_information_activate (GtkAction *action, gpointer data);
 static void on_network_reconnect_activate (GtkAction *action, gpointer data);
 static void on_network_disconnect_activate (GtkAction *action, gpointer data);
 static void on_network_close_activate (GtkAction *action, gpointer data);
@@ -116,7 +115,6 @@ static GtkActionEntry action_entries [] = {
 	{ "EditPreferences", GTK_STOCK_PREFERENCES, N_("Prefere_nces"), "",           NULL, G_CALLBACK (on_edit_preferences_activate) },
 
 	/* Network menu */
-	{ "NetworkInformation", GTK_STOCK_DIALOG_INFO, N_("_Information"), "",                  NULL, G_CALLBACK (on_network_information_activate) },
 	{ "NetworkReconnect",   GTK_STOCK_REFRESH,     N_("_Reconnect"),   "<control>R",        NULL, G_CALLBACK (on_network_reconnect_activate) },
 	{ "NetworkDisconnect",  GTK_STOCK_DISCONNECT,  N_("_Disconnect"),  "",                  NULL, G_CALLBACK (on_network_disconnect_activate) },
 	{ "NetworkClose",       GTK_STOCK_CLOSE,       N_("_Close"),       "<shift><control>W", NULL, G_CALLBACK (on_network_close_activate) },
@@ -322,15 +320,13 @@ run_main_window ()
 	g_signal_connect (G_OBJECT (pane), "notify::position", G_CALLBACK (on_hpane_move), NULL);
 	g_object_unref (client);
 
-	gtk_widget_show (gui.main_window);
-
 	/* Temporarily disable menu items */
-	widget = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/NetworkMenu/NetworkInformationItem");
-	gtk_widget_set_sensitive (widget, FALSE);
 	widget = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/DiscussionMenu/DiscussionBansItem");
 	gtk_widget_set_sensitive (widget, FALSE);
-	widget = gtk_ui_manager_get_widget (gui.manager, "/ui/ChannelPopup/ChannelBans");
+	widget = gtk_ui_manager_get_widget (gui.manager, "/ui/ChannelJoinedPopup/ChannelBans");
 	gtk_widget_set_sensitive (widget, FALSE);
+
+	gtk_widget_show (gui.main_window);
 }
 
 void
@@ -433,12 +429,6 @@ on_edit_preferences_activate (GtkAction *action, gpointer data)
 	preferences_dialog_show (gui.prefs_dialog);
 }
 
-static void
-on_network_information_activate (GtkAction *action, gpointer data)
-{
-	/* FIXME: implement */
-}
-
 static void
 on_network_reconnect_activate (GtkAction *action, gpointer data)
 {
@@ -502,7 +492,7 @@ static void
 on_discussion_leave_activate (GtkAction *action, gpointer data)
 {
 	session *s = gui.current_session;
-	if ((s->type == SESS_CHANNEL) && (s->channel[0] != '\0')) {
+	if ((s != NULL) && (s->type == SESS_CHANNEL) && (s->channel[0] != '\0')) {
 		gchar *text;
 		GConfClient *client;
 
@@ -751,6 +741,7 @@ set_nickname_color (struct server *serv)
 static gboolean
 on_main_window_focus_in (GtkWidget * widget, GdkEventFocus * event, gpointer data)
 {
+	conversation_panel_check_marker_visibility (CONVERSATION_PANEL (gui.conversation_panel));
 	gtk_window_set_urgency_hint (GTK_WINDOW (widget), FALSE);
 	return FALSE;
 }
