@@ -73,6 +73,7 @@ static void      on_close (GtkAction *action, gpointer data);
 static void      on_find (GtkAction *action, gpointer data);
 static void      on_channel_leave (GtkAction *action, gpointer data);
 static void      on_channel_bans (GtkAction *action, gpointer data);
+static void      on_channel_join (GtkAction *action, gpointer data);
 static void      on_server_autoconnect (GtkAction *action, gpointer data);
 static void      on_channel_autojoin (GtkAction *action, gpointer data);
 
@@ -88,6 +89,7 @@ static GtkActionEntry action_entries[] = {
 	{"ChannelSave",       GTK_STOCK_SAVE,           N_("_Save Transcript"), "", NULL, G_CALLBACK (on_save)},
 	{"ChannelLeave",      GTK_STOCK_QUIT,           N_("_Leave"),           "", NULL, G_CALLBACK (on_channel_leave)},
 	{"ChannelClose",      GTK_STOCK_CLOSE,          N_("_Close"),           "", NULL, G_CALLBACK (on_close)},
+	{"ChannelJoin",       GTK_STOCK_JUMP_TO,        N_("_Join"),            "", NULL, G_CALLBACK (on_channel_join)},
 	{"ChannelFind",       GTK_STOCK_FIND,           N_("_Find..."),         "", NULL, G_CALLBACK (on_find)},
 	{"ChannelBans",       GTK_STOCK_DIALOG_WARNING, N_("_Bans..."),         "", NULL, G_CALLBACK (on_channel_bans)},
 
@@ -949,7 +951,11 @@ channel_context (GtkWidget *treeview, session *selected)
 	GtkAction *action;
 	ircnet *network;
 
-	menu = gtk_ui_manager_get_widget (gui.manager, "/ChannelPopup");
+	if (selected->total) {
+		menu = gtk_ui_manager_get_widget (gui.manager, "/ChannelJoinedPopup");
+	} else {
+		menu = gtk_ui_manager_get_widget (gui.manager, "/ChannelUnjoinedPopup");
+	}
 	g_return_val_if_fail (menu != NULL, NULL);
 
 	/* check if the channel is in the auto-join list */
@@ -1679,6 +1685,28 @@ on_channel_bans (GtkAction * action, gpointer data)
 {
 }
 
+static void
+on_channel_join (GtkAction *action, gpointer data)
+{
+	GtkTreeView *treeview;
+	GtkTreeSelection *select;
+	GtkTreeModel *model, *store;
+	GtkTreeIter iter;
+	gchar *channel;
+	session *s;
+
+	treeview = GTK_TREE_VIEW (gui.server_tree);
+	select = gtk_tree_view_get_selection (treeview);
+	if (gtk_tree_selection_get_selected (select, &model, &iter)) {
+		gtk_tree_model_get (model, &iter, 1, &channel, 2, &s, -1);
+		store = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (model));
+		if ((s->type == SESS_CHANNEL)) {
+			s->server->p_join(s->server, channel, "");
+		}
+		g_free (channel);
+	}
+}
+
 static void
 on_server_autoconnect (GtkAction *action, gpointer data)
 {
@@ -1711,6 +1739,8 @@ on_channel_autojoin (GtkAction *action, gpointer data)
 	session *sess;
 	gboolean autojoin, active;
 	ircnet *network;
+	gchar **autojoins = NULL;
+	gchar *tmp;
 
 	sess = navigation_tree_get_selected_session (NULL);
 	network = sess->server->network;
@@ -1720,61 +1750,106 @@ on_channel_autojoin (GtkAction *action, gpointer data)
 	if (sess != NULL) {
 		if (autojoin && !active) {
 			/* remove channel from autojoin list */
-			gchar **channels, *tmp;
+			gchar **channels, **keys;
+			gchar *new_channels, *new_keys;
 			gint i;
+			gboolean keys_done;
 
-			channels = g_strsplit (network->autojoin, ",", 0);
+			new_channels = new_keys = NULL;
+			autojoins = g_strsplit (network->autojoin, " ", 0);
+			channels = g_strsplit (autojoins[0], ",", 0);
 
-			if (network->autojoin) g_free(network->autojoin);
+			if (autojoins[1]) {
+				keys =  g_strsplit (autojoins[1], ",", 0);
+				keys_done = FALSE;
+			} else {
+				keys = NULL;
+				keys_done = TRUE;
+			}
+
+			g_free (network->autojoin);
 			network->autojoin = NULL;
 
 			for (i=0; channels[i]; i++) {
 				if (strcmp (channels[i], sess->channel) != 0) {
-					if ( network->autojoin == NULL)
-						network->autojoin = g_strdup (channels[i]);
+					if (new_channels == NULL)
+						new_channels = g_strdup (channels[i]);
 					else {
-						tmp = network->autojoin;
-						network->autojoin = g_strdup_printf ("%s,%s", network->autojoin, channels[i]);
+						tmp = new_channels;
+						new_channels = g_strdup_printf ("%s,%s", new_channels, channels[i]);
 						g_free (tmp);
 					}
+
+					if (!keys_done && keys[i]) {
+						if (new_keys == NULL)
+							new_keys = g_strdup (keys[i]);
+						else {
+							tmp = new_keys;
+							new_keys = g_strdup_printf ("%s,%s", new_keys, keys[i]);
+							g_free (tmp);
+						}
+					}
+					else
+						keys_done = TRUE;
 				}
+				else if (!keys_done && !keys[i])
+					keys_done = TRUE;
 			}
-			g_strfreev (channels);
+
+			if (new_keys) {
+				network->autojoin = g_strdup_printf ("%s %s", new_channels, new_keys);
+				g_free (new_channels);
+				g_free (new_keys);
+			}
+			else
+				network->autojoin = new_channels;
 
 			servlist_save ();
+			g_strfreev (channels);
+			if (keys) g_strfreev (keys);
 		}
+
 		if (!autojoin && active) {
 			/* add channel to autojoin list */
+			/* FIXME: we should save the key of the channel is there is one */
 			if (network->autojoin == NULL) {
 				network->autojoin = g_strdup (sess->channel);
 			}
 			else {
-				gchar *tmp = network->autojoin;
+				autojoins = g_strsplit (network->autojoin, " ", 0);
+				tmp = network->autojoin;
+
+				if (autojoins[1])
+					network->autojoin = g_strdup_printf ("%s,%s %s", autojoins[0], sess->channel, autojoins[1]);
+				else
+					network->autojoin = g_strdup_printf ("%s,%s", autojoins[0], sess->channel);
 
-				network->autojoin = g_strdup_printf ("%s,%s", network->autojoin, sess->channel);
 				g_free (tmp);
 			}
 
 			servlist_save ();
 		}
+		
+		if (autojoins) g_strfreev (autojoins);
 	}
-
 }
 
 static gboolean
 channel_is_autojoin (ircnet *network, gchar *chan)
 {
-	gchar **channels;
+	gchar **autojoin, **channels;
 	gint i;
 	gboolean find = FALSE;
 
 	if (network->autojoin == NULL) return FALSE;
 
-	channels = g_strsplit (network->autojoin, ",", 0);
+	autojoin = g_strsplit (network->autojoin, " ", 0);
+	channels = g_strsplit (autojoin[0], ",", 0);
 	for (i=0; channels[i] && !find; i++)
 		if (!strcmp (channels[i], chan))
 			find = TRUE;
 
+	g_strfreev (autojoin);
 	g_strfreev (channels);
 
 	return find;
