@@ -22,12 +22,18 @@
 #include <config.h>
 #include <glib/gi18n.h>
 #include <gnome.h>
+#include <string.h>
+#ifdef HAVE_LIBSEXY
+#include <libsexy/sexy-tree-view.h>
+#endif
 #include "userlist-gui.h"
 #include "pixmaps.h"
 #include "palette.h"
 #include "../common/xchat.h"
 #include "../common/userlist.h"
 #include "../common/outbound.h"
+#include "../common/util.h"
+#include "../common/server.h"
 
        gboolean userlist_click              (GtkWidget *view, GdkEventButton *event, gpointer data);
        void     userlist_context            (GtkWidget *treeview, struct User *user);
@@ -38,6 +44,10 @@ static gboolean userlist_window_grab_broken (GtkWidget *window, GdkEventGrabBrok
 static void     userlist_popup_deactivate   (GtkMenuShell *menu, gpointer data);
 static gboolean userlist_button_release     (GtkWidget *widget, GdkEventButton *button, gpointer data);
 struct User*    userlist_get_selected       (void);
+#ifdef HAVE_LIBSEXY
+GtkWidget* get_user_vbox_infos	    	    (struct User *user);
+static GtkWidget* get_user_tooltip	    (SexyTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *column, gpointer data);
+#endif
 
 /* action callbacks */
 static void user_send_file_activate   (GtkAction *action, gpointer data);
@@ -45,13 +55,15 @@ static void user_open_dialog_activate (GtkAction *action, gpointer data);
 static void user_kick_activate        (GtkAction *action, gpointer data);
 static void user_ban_activate         (GtkAction *action, gpointer data);
 static void user_ignore_activate      (GtkAction *action, gpointer data);
+static void user_op_activate          (GtkAction *action, gpointer data);
 
-static GtkActionEntry popup_action_entries [] = {
+static GtkActionEntry popup_action_entries[] = {
 	{ "UserlistSendFile",   NULL, N_("_Send File..."), "",   NULL, G_CALLBACK (user_send_file_activate) },
 	{ "UserlistOpenDialog", NULL, N_("Private _Chat"), "",   NULL, G_CALLBACK (user_open_dialog_activate) },
 	{ "UserlistKick",       NULL, N_("_Kick"),         "",   NULL, G_CALLBACK (user_kick_activate) },
 	{ "UserlistBan",        NULL, N_("_Ban"),          "",   NULL, G_CALLBACK (user_ban_activate) },
 	{ "UserlistIgnore",     NULL, N_("Ignore"),        "",   NULL, G_CALLBACK (user_ignore_activate) },
+	{ "UserlistOp",	        NULL, N_("_Op"),           "",   NULL, G_CALLBACK (user_op_activate) },
 };
 
 struct User     *current_user;
@@ -64,10 +76,22 @@ initialize_userlist (void)
 	GtkCellRenderer *icon_renderer, *text_renderer;
 	GtkTreeViewColumn *icon_column, *text_column;
 	GtkTreeSelection *select;
+	GtkWidget *swin;
 
-	gui.userlist        = glade_xml_get_widget (gui.xml, "userlist");
+#ifdef HAVE_LIBSEXY
+	gui.userlist = sexy_tree_view_new ();
+#else
+	gui.userlist = gtk_tree_view_new ();
+#endif
+	gtk_widget_show (gui.userlist);
 	gui.userlist_window = glade_xml_get_widget (gui.xml, "userlist_window");
 
+	swin = glade_xml_get_widget (gui.xml, "scrolledwindow_userlist");
+	gtk_container_add (GTK_CONTAINER (swin), gui.userlist);
+
+	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (gui.userlist), FALSE);
+	gtk_tree_view_set_hover_selection (GTK_TREE_VIEW (gui.userlist), TRUE);
+
 	icon_renderer = gtk_cell_renderer_pixbuf_new ();
 	icon_column = gtk_tree_view_column_new_with_attributes ("icon", icon_renderer, "pixbuf", 0, NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (gui.userlist), icon_column);
@@ -82,6 +106,9 @@ initialize_userlist (void)
 	g_signal_connect (G_OBJECT (gui.userlist_window), "button_release_event", G_CALLBACK (userlist_button_release), NULL);
 	g_signal_connect (G_OBJECT (gui.userlist_window), "grab_broken_event", G_CALLBACK (userlist_window_grab_broken), NULL);
 	g_signal_connect (G_OBJECT (gui.userlist_window), "event", G_CALLBACK (userlist_window_event), NULL);
+#ifdef HAVE_LIBSEXY
+	g_signal_connect(G_OBJECT (gui.userlist), "get-tooltip", G_CALLBACK (get_user_tooltip), NULL);
+#endif
 
 	gtk_action_group_add_actions (gui.action_group, popup_action_entries, G_N_ELEMENTS (popup_action_entries), NULL);
 }
@@ -209,16 +236,27 @@ user_ignore_activate (GtkAction *action, gpointer data)
 	userlist_gui_hide ();
 }
 
+static void
+user_op_activate (GtkAction *action, gpointer data)
+{
+	user_cmd ("op", current_user->nick);
+
+	userlist_gui_hide ();
+}
+
 void
 userlist_gui_show (void)
 {
-	gint screen_width, screen_height;
-	gint desired_height;
-	gint window_x, window_y;
-	gint  mouse_x,  mouse_y;
-	GdkDisplay *display;
-	GdkScreen *mouse_screen;
-	GtkRequisition request;
+	gint            desired_height;
+	gint            window_x;
+	gint            window_y;
+	gint            mouse_x;
+	gint            mouse_y;
+	gint            monitor;
+	GdkRectangle    monitor_rect;
+	GdkDisplay     *display;
+	GdkScreen      *mouse_screen;
+	GtkRequisition  request;
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gui.userlist_toggle), TRUE);
 
@@ -229,27 +267,27 @@ userlist_gui_show (void)
 	display = gdk_display_get_default ();
 	gdk_display_get_pointer (display, &mouse_screen, &mouse_x, &mouse_y, NULL);
 
-	screen_width  = gdk_screen_get_width  (mouse_screen);
-	screen_height = gdk_screen_get_height (mouse_screen);
+	monitor = gdk_screen_get_monitor_at_point (mouse_screen, mouse_x, mouse_y);
+	gdk_screen_get_monitor_geometry (mouse_screen, monitor, &monitor_rect);
 
 	/* Buffer of 20 pixels.  Would be nice to know exactly how much space
 	 * the rest of the window's UI goop used up, but oh well.
 	 */
 	desired_height = request.height + 20;
-	if (desired_height > screen_height)
-		desired_height = screen_height;
+	if (desired_height > monitor_rect.height)
+		desired_height = monitor_rect.height;
 
-	window_x = mouse_x - 100;
+	window_x = mouse_x + 10;
 	window_y = mouse_y - (desired_height / 2);
 
-	if (window_x < 0)
-		window_x = 0;
-	if (window_x + 250 > screen_width)
-		window_x = screen_width - 250;
-	if (window_y < 0)
-		window_y = 0;
-	if (window_y + desired_height > screen_height)
-		window_y = screen_height - desired_height;
+	if (window_x < monitor_rect.x)
+		window_x = monitor_rect.x;
+	if (window_x + 250 > monitor_rect.width)
+		window_x = monitor_rect.width - 250;
+	if (window_y < monitor_rect.y)
+		window_y = monitor_rect.y;
+	if (window_y + desired_height > monitor_rect.height)
+		window_y = monitor_rect.height - desired_height;
 	gtk_window_move (GTK_WINDOW (gui.userlist_window), window_x, window_y);
 
 	gtk_window_resize (GTK_WINDOW (gui.userlist_window), 250, desired_height);
@@ -261,6 +299,8 @@ userlist_gui_show (void)
 void
 userlist_gui_hide (void)
 {
+	gint position;
+
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gui.userlist_toggle), FALSE);
 	if (have_grab) {
 		gtk_grab_remove (gui.userlist_window);
@@ -270,7 +310,9 @@ userlist_gui_hide (void)
 	}
 	gtk_widget_hide (gui.userlist_window);
 
+	position = gtk_editable_get_position (GTK_EDITABLE (gui.text_entry));
 	gtk_widget_grab_focus (gui.text_entry);
+	gtk_editable_set_position (GTK_EDITABLE (gui.text_entry), position);
 }
 
 static gboolean
@@ -344,3 +386,73 @@ userlist_button_release (GtkWidget *widget, GdkEventButton *button, gpointer dat
 	userlist_gui_hide ();
 	return TRUE;
 }
+
+#ifdef HAVE_LIBSEXY
+GtkWidget*
+get_user_vbox_infos (struct User *user)
+{
+	GtkWidget *vbox, *label;
+	gchar *text, *tmp, *country_txt;
+
+	vbox = gtk_vbox_new (FALSE, 0);
+
+	text = g_strdup_printf ("<span size=\"large\" weight=\"bold\">%s</span>", user->nick);
+
+	if (user->realname && strlen (user->realname) > 0) {
+		tmp = text;
+		text = g_strdup_printf (_("%s\n<span weight=\"bold\">Name:</span> %s"), text, user->realname);
+		g_free (tmp);
+	}
+
+	country_txt = country (user->hostname);
+	if (country_txt && strcmp (country_txt, _("Unknown")) != 0) {
+		tmp = text;
+		text = g_strdup_printf (_("%s\n<span weight=\"bold\">Country:</span> %s"), text, country_txt);
+		g_free (tmp);
+	}
+
+	if (user->lasttalk) {
+		gint last;
+
+		last = (gint) (time (NULL) - user->lasttalk) / 60;
+		if (last >= 1) {
+			tmp = text;
+			text = g_strdup_printf (ngettext("%s\n<span weight=\"bold\">Last message:</span> %d minute ago", "%s\n<span weight=\"bold\">Last message:</span> %d minutes ago", last), text, last);
+			g_free (tmp);
+		}
+	}
+
+	if (user->away) {
+		struct away_msg *away_msg;
+
+		away_msg = server_away_find_message (gui.current_session->server, user->nick);
+		if (away_msg) {
+			tmp = text;
+			text =  g_strdup_printf (_("%s\n<span weight=\"bold\">Away message:</span> %s"), text, away_msg->message);
+			g_free (tmp);
+		}
+	}
+
+	label = gtk_label_new (NULL);
+	gtk_label_set_markup (GTK_LABEL (label), text);
+	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 0);
+	g_free (text);
+
+	gtk_widget_show_all (vbox);
+
+	return vbox;
+}
+
+static GtkWidget*
+get_user_tooltip (SexyTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *column, gpointer data)
+{
+	struct User *user;
+
+	if (GTK_WIDGET_VISIBLE (gui.userlist_window) == FALSE)
+		return NULL;
+
+	user = userlist_get_selected ();
+
+	return get_user_vbox_infos (user);
+}
+#endif
