@@ -28,5 +28,7 @@ void error_dialog (const gchar *header, const gchar *message);
 gint gtk_tree_iter_sort_func_nocase (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer data);
 gboolean dialog_escape_key_handler_destroy (GtkWidget *widget, GdkEventKey *event, gpointer data);
 gboolean dialog_escape_key_handler_hide (GtkWidget *widget, GdkEventKey *event, gpointer data);
+void menu_position_under_widget (GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer user_data);
+void menu_position_under_tree_view (GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer user_data);
 
 #endif
